//
//  ISVehicleCardResultViewController.m
//  ISVehicleCardCameraPreview
//
//  Created by 汪凯 on 15/11/13.
//  Copyright © 2015年 汪凯. All rights reserved.
//

#import "ISVehicleCardResultViewController.h"
#import <ISVehicleLicensePreviewSDK/ISVehicleLicensePreviewSDK.h>

#define UIColorFromRGB(rgbValue) \
[UIColor \
colorWithRed:((float)((rgbValue & 0xFF0000) >> 16))/255.0 \
green:((float)((rgbValue & 0x00FF00) >> 8))/255.0 \
blue:((float)(rgbValue & 0x0000FF))/255.0 \
alpha:1.0]

@implementation ISVehicleCardResultItem

-(id) init
{
    self = [super init];
    if (self) {
        _typeText = nil;
        _resultText = nil;
        _numberOfline = 1;
    }
    return self;
}

@end

@interface ISVehicleCardResultViewController ()<UITableViewDataSource, UITableViewDelegate>

@property (weak, nonatomic) IBOutlet UIImageView *bgImageView;

@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic) IBOutlet UIButton *doneButton;

@property (strong, nonatomic) UILabel *titleLabel;
@property (strong, nonatomic) IBOutlet UIView *tableHeaderView;
@property (strong, nonatomic) UIImageView *vehicleLicenseImageView;

@property (strong, nonatomic) NSDictionary *infoDic;
@property (strong, nonatomic) NSMutableArray *dataSource;
@property (nonatomic, strong) UIImage *originImage;
@property (nonatomic, strong) UIImage *cropImage;
@property (nonatomic, strong) NSMutableArray *croppedItemImageArray;

@end

@implementation ISVehicleCardResultViewController

- (instancetype)initWithDictionary:(NSDictionary *)info
{
    self = [super init];
    if (self != nil)
    {
        _infoDic = info;
        _croppedItemImageArray = [NSMutableArray array];
        
    }
    return self;
}

- (IBAction)doneButtonDidClicked:(id)sender
{
    [self dismissViewControllerAnimated:YES completion:^
     {
         
     }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    CGSize ScreenSize = [[UIScreen mainScreen] bounds].size;
    self.bgImageView.image = [UIImage imageNamed:@"bg"];
    
//    self.tableHeaderView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, ScreenSize.width, 174)];
    self.tableHeaderView.backgroundColor = UIColorFromRGB(0x13334A);
    
    self.titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 34, ScreenSize.width, 20)];
    self.titleLabel.textColor = [UIColor whiteColor];
    self.titleLabel.textAlignment = NSTextAlignmentCenter;
    [self.tableHeaderView addSubview:self.titleLabel];
    
    self.vehicleLicenseImageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 64, ScreenSize.width, 300-64)];
    self.vehicleLicenseImageView.contentMode = UIViewContentModeScaleAspectFit;
    [self.tableHeaderView addSubview:self.vehicleLicenseImageView];
    
//    self.tableView.tableHeaderView = self.tableHeaderView;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.tableView.backgroundColor = [UIColor clearColor];
    
    [self.doneButton setBackgroundColor:UIColorFromRGB(0x18B8F5)];
    self.doneButton.layer.cornerRadius = 4.0f;
    [self setupDataSource];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setupDataSource
{
    CGSize screenSize = [[UIScreen mainScreen] bounds].size;
    {
        UIImage *vehicleLicenseImage = [self.infoDic valueForKey:kOpenSDKCardResultTypeImage];
        self.cropImage = vehicleLicenseImage;
        self.vehicleLicenseImageView.image = vehicleLicenseImage;
        self.originImage = [self.infoDic valueForKey:kOpenSDKCardResultTypeOriginImage];
        CGFloat imageHeight = (screenSize.width-20)/vehicleLicenseImage.size.width*vehicleLicenseImage.size.height;
        self.vehicleLicenseImageView.frame = CGRectMake(10, 64, screenSize.width-20, imageHeight);
//        self.tableHeaderView.frame = CGRectMake(0, 0, screenSize.width, CGRectGetMaxY(self.vehicleLicenseImageView.frame)+20);
        for (NSLayoutConstraint* constraint in self.tableHeaderView.constraints)
        {
            if (constraint.firstAttribute == NSLayoutAttributeHeight)
            {
                constraint.constant = CGRectGetMaxY(self.vehicleLicenseImageView.frame)+10;
            }
        }
    }
    
    self.titleLabel.text = NSLocalizedString(@"行驶证识别", @"行驶证识别");
    [self.doneButton setTitle:NSLocalizedString(@"确定", @"确定") forState:UIControlStateNormal];
    
    self.dataSource = [NSMutableArray array];
    
    NSDictionary *itemInfo = [self.infoDic valueForKey:kOpenSDKCardResultTypeCardItemInfo];
    
    {
        ISCardReaderResultItem *plateNumber = [itemInfo valueForKey:kVehicleCardItemPlateNumber];
        if (plateNumber) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Plate Number:", @"号牌号码:");
            item.resultText = plateNumber.itemValue;
            NSArray *pointsInOriginImage = plateNumber.pointsInOriginImage;
            CGRect rectInCroppedImage = plateNumber.rectInCroppedImage;
            NSLog(@"plateNumber.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:plateNumber.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *vehicleType = [itemInfo valueForKey:kVehicleCardItemVehicleType];
        if (vehicleType) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Vehicle Type:", @"车辆类型:");
            item.resultText = vehicleType.itemValue;
            NSArray *pointsInOriginImage = vehicleType.pointsInOriginImage;
            CGRect rectInCroppedImage = vehicleType.rectInCroppedImage;
            NSLog(@"vehicleType.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:vehicleType.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *Owner = [itemInfo valueForKey:kVehicleCardItemOwner];
        if (Owner) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Owner:", @"所有人:");
            item.resultText = Owner.itemValue;
            NSArray *pointsInOriginImage = Owner.pointsInOriginImage;
            CGRect rectInCroppedImage = Owner.rectInCroppedImage;
            NSLog(@"Owner.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:Owner.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *address = [itemInfo valueForKey:kVehicleCardItemAddress];
        if (address) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Address:", @"地址:");
            item.resultText = address.itemValue;
            item.numberOfline = 2;
            NSArray *pointsInOriginImage = address.pointsInOriginImage;
            CGRect rectInCroppedImage = address.rectInCroppedImage;
            NSLog(@"address.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:address.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *useProperty = [itemInfo valueForKey:kVehicleCardItemUseProperty];
        if (useProperty) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Use Property:", @"使用性质:");
            item.resultText = useProperty.itemValue;
            NSArray *pointsInOriginImage = useProperty.pointsInOriginImage;
            CGRect rectInCroppedImage = useProperty.rectInCroppedImage;
            NSLog(@"useProperty.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:useProperty.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *brandModel = [itemInfo valueForKey:kVehicleCardItemBrandModel];
        if (brandModel) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Brand Model:", @"品牌型号:");
            item.resultText = brandModel.itemValue;
            NSArray *pointsInOriginImage = brandModel.pointsInOriginImage;
            CGRect rectInCroppedImage = brandModel.rectInCroppedImage;
            NSLog(@"brandModel.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:brandModel.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }

    {
        ISCardReaderResultItem *itemVIN = [itemInfo valueForKey:kVehicleCardItemVIN];
        if (itemVIN) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Item VIN:", @"车辆识别代号:");
            item.resultText = itemVIN.itemValue;
            NSArray *pointsInOriginImage = itemVIN.pointsInOriginImage;
            CGRect rectInCroppedImage = itemVIN.rectInCroppedImage;
            NSLog(@"itemVIN.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:itemVIN.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *engineNumber = [itemInfo valueForKey:kVehicleCardItemEngineNumber];
        if (engineNumber) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Engine Number:", @"发动机号码:");
            item.resultText = engineNumber.itemValue;
            NSArray *pointsInOriginImage = engineNumber.pointsInOriginImage;
            CGRect rectInCroppedImage = engineNumber.rectInCroppedImage;
            NSLog(@"engineNumber.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:engineNumber.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *registerDate = [itemInfo valueForKey:kVehicleCardItemRegisterDate];
        if (registerDate) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Register Date:", @"注册日期:");
            item.resultText = registerDate.itemValue;
            NSArray *pointsInOriginImage = registerDate.pointsInOriginImage;
            CGRect rectInCroppedImage = registerDate.rectInCroppedImage;
            NSLog(@"registerDate.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:registerDate.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
    
    {
        ISCardReaderResultItem *issueDate = [itemInfo valueForKey:kVehicleCardItemIssueDate];
        if (issueDate) {
            ISVehicleCardResultItem *item = [[ISVehicleCardResultItem alloc] init];
            item.typeText = NSLocalizedString(@"Issue Date:", @"发证日期:");
            item.resultText = issueDate.itemValue;
            NSArray *pointsInOriginImage = issueDate.pointsInOriginImage;
            CGRect rectInCroppedImage = issueDate.rectInCroppedImage;
            NSLog(@"issueDate.pointsInOriginImage: %@, rect: %@", pointsInOriginImage, [NSValue valueWithCGRect:rectInCroppedImage]);
            [self.croppedItemImageArray addObject:issueDate.itemCroppendImage];
            [self.dataSource addObject:item];
        }
    }
}

#pragma mark - UITableViewDataSource & UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.dataSource.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const ISVehicleLicenseResultTableViewCellIdentifier = @"ISVehicleLicenseResultTableViewCellIdentifier";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:ISVehicleLicenseResultTableViewCellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:ISVehicleLicenseResultTableViewCellIdentifier];
        cell.backgroundColor = [UIColor clearColor];
        cell.contentView.backgroundColor = [UIColor clearColor];
        cell.textLabel.textColor = UIColorFromRGB(0xE1E1E1);
        cell.detailTextLabel.textColor = UIColorFromRGB(0xFFFFFF);
        cell.detailTextLabel.textAlignment = NSTextAlignmentRight;
        cell.detailTextLabel.adjustsFontSizeToFitWidth = YES;
        cell.detailTextLabel.minimumScaleFactor = 0.5f;
        cell.detailTextLabel.clipsToBounds = YES;

    }
    
    if (indexPath.row < self.dataSource.count) {
        NSObject *data = self.dataSource[indexPath.row];
        if ([data isKindOfClass:[NSString class]]) {
            cell.textLabel.text = (NSString *)data;
        } else if ([data isKindOfClass:[ISVehicleCardResultItem class]]) {
            ISVehicleCardResultItem *item = (ISVehicleCardResultItem*)data;
            cell.detailTextLabel.numberOfLines = item.numberOfline;
            cell.textLabel.text = item.typeText;
            cell.detailTextLabel.text = item.resultText;
        }
    }
    
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
//    NSUInteger row = indexPath.row;
//    if (row < self.dataSource.count) {
//        ISVehicleCardResultItem *item = self.dataSource[row];
//        if (item.numberOfline <= 1) {
//            return 44.0;
//        }
//        else
//        {
//            return 44.0 + (item.numberOfline -1) * 30;
//        }
//    }
    return 44.0;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}


- (UIImage *) croppedImage:(UIImage*)image withRect: (CGRect) bounds
{
    CGImageRef imageRef = CGImageCreateWithImageInRect([image CGImage], bounds);
    UIImage *croppedImage = [UIImage imageWithCGImage:imageRef];
    CGImageRelease(imageRef);
    return croppedImage;
}

@end
