//
//  ViewController.m
//  ISVehicleLicensePreviewDemo
//
//  Created by Johnson Zhang on 16/4/20.
//  Copyright © 2016年 Johnson Zhang. All rights reserved.
//

#import "ViewController.h"
#import <AVFoundation/AVFoundation.h>
#import <ISVehicleLicensePreviewSDK/ISVehicleLicensePreviewSDK.h>
#import <ISOpenSDKFoundation/ISOpenSDKFoundation.h>
#import <CoreGraphics/CoreGraphics.h>
#import "MBProgressHUD.h"
#import "ISVehicleCardResultViewController.h"

#define PREVIEW_BORDER_WIDTH 270
#define PREVIEW_BORDER_HEIGHT 428

typedef NS_ENUM(NSInteger, ISPreviewExpandOrientation)
{
    ISPreviewExpandOrientationPortait,
    ISPreviewExpandOrientationLandscape
};

static const NSString *AVCaptureStillImageIsCapturingStillImageContext = @"AVCaptureStillImageIsCapturingStillImageContext";

@interface ViewController ()<AVCaptureVideoDataOutputSampleBufferDelegate, UIAlertViewDelegate>

@property (nonatomic, strong) AVCaptureSession *captureSession;
@property (nonatomic, strong) AVCaptureVideoPreviewLayer *previewLayer;
@property (nonatomic, strong) AVCaptureVideoDataOutput *videoDataOutput;
@property (nonatomic, strong) dispatch_queue_t videoDataOutputQueue;
@property (nonatomic, strong) AVCaptureStillImageOutput *stillImageOutput;
@property (nonatomic, strong) UIView *borderView;
@property (nonatomic, assign) int index;
@property (nonatomic, strong) CALayer *cardNumberLayer;
@property (nonatomic, strong) CALayer *cardBorderLayer;

@property (nonatomic, assign) CGFloat imageFrameWidth;
@property (nonatomic, assign) CGFloat imageFrameHeight;
@property (nonatomic, assign) ISPreviewExpandOrientation expandOrientation;

@end

@implementation ViewController

- (void) dealloc
{
    [[ISVehicleLicensePreviewController sharedISOpenSDKController] destructResources];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^
                   {
                       //#error Please fill in your appKey
                       NSString *appKey = @"a79c19499739adb69caf2bf851-vagfvt";
                       NSString *subAppkey = nil;//reserved for future use
                       [[ISVehicleLicensePreviewController sharedISOpenSDKController] constructResourcesWithAppKey:appKey subAppkey:subAppkey finishHandler:^(ISOpenSDKStatus status) {
                           if(status == ISOpenSDKStatusSuccess)
                           {
                               dispatch_async(dispatch_get_main_queue(), ^
                                              {
                                                  [self setupAVCapture];
                                                  [self setupBorderView];
                                                  [self setupOtherThings];
                                                  [self focus];
                                                  if (![self.captureSession isRunning])
                                                  {
                                                      [self.captureSession startRunning];
                                                  }
                                              });
                           }
                           else
                           {
                               NSLog(@"Authorize error");
                           }
                       }];
                   });
}

- (void) focus
{
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    if(device.isFocusPointOfInterestSupported && [device isFocusModeSupported:AVCaptureFocusModeContinuousAutoFocus])
    {
        NSError *error =  nil;
        if([device lockForConfiguration:&error]){
            [device setFocusPointOfInterest:CGPointMake(.5f, .5f)];
            [device setFocusMode:AVCaptureFocusModeContinuousAutoFocus];
            [device unlockForConfiguration];
        }
    }
}

//设置视频流捕获
- (void)setupAVCapture
{
    NSError *error = nil;
    
    AVCaptureSession *session = [AVCaptureSession new];
    //1920*1080 is the suggested size if it is supported by device.
    if ([session canSetSessionPreset:AVCaptureSessionPreset1920x1080])
    {
        NSLog(@"AVCaptureSessionPreset1920x1080");
        [session setSessionPreset:AVCaptureSessionPreset1920x1080];
        self.imageFrameWidth = 1920;
        self.imageFrameHeight = 1080;
    }
    else if ([session canSetSessionPreset:AVCaptureSessionPreset640x480])
    {
        NSLog(@"AVCaptureSessionPreset640x480");
        [session setSessionPreset:AVCaptureSessionPreset640x480];
        self.imageFrameWidth = 640;
        self.imageFrameHeight = 480;
    }
    else if ([session canSetSessionPreset:AVCaptureSessionPreset352x288])
    {
        NSLog(@"AVCaptureSessionPreset352x288");
        [session setSessionPreset:AVCaptureSessionPreset352x288];
        self.imageFrameWidth = 352;
        self.imageFrameHeight = 288;
    }
    else
    {
        NSLog(@"failed  setSessionPreset");
        return;
    }
    self.captureSession = session;
    // Select a video device, make an input
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    AVCaptureDeviceInput *deviceInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
    
    if ( [session canAddInput:deviceInput] )
        [session addInput:deviceInput];
    
    // Make a video data output
    self.videoDataOutput = [AVCaptureVideoDataOutput new];
    
    // we want YUV, both CoreGraphics and OpenGL work well with 'YUV'
    NSDictionary *rgbOutputSettings = [NSDictionary dictionaryWithObject:
                                       [NSNumber numberWithInt:kCVPixelFormatType_420YpCbCr8BiPlanarFullRange] forKey:(id)kCVPixelBufferPixelFormatTypeKey];
    [self.videoDataOutput setVideoSettings:rgbOutputSettings];
    [self.videoDataOutput setAlwaysDiscardsLateVideoFrames:YES];
    
    self.videoDataOutputQueue = dispatch_queue_create("VideoDataOutputQueue", DISPATCH_QUEUE_SERIAL);
    [self.videoDataOutput setSampleBufferDelegate:self queue:self.videoDataOutputQueue];
    
    if ([session canAddOutput:self.videoDataOutput])
    {
        [session addOutput:self.videoDataOutput];
    }
    [[self.videoDataOutput connectionWithMediaType:AVMediaTypeVideo] setEnabled:YES];
    
    //屏幕成像预览区域
    self.previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:session];
    [self.previewLayer setBackgroundColor:[[UIColor blackColor] CGColor]];
    //设置填充模式
    [self.previewLayer setVideoGravity:AVLayerVideoGravityResizeAspect];
    
    CALayer *rootLayer = [self.view layer];
    [rootLayer setMasksToBounds:YES];
    
    //获取当前屏幕尺寸大小
    CGFloat screenWidth = [[UIScreen mainScreen] bounds].size.width;
    CGFloat scrrenHeight = [[UIScreen mainScreen] bounds].size.height;
    
    
    //建立视频预览区域大小
    if (screenWidth / self.imageFrameHeight > scrrenHeight / self.imageFrameWidth)
    {
        CGFloat height = self.imageFrameWidth * screenWidth / self.imageFrameHeight;
        CGRect frameRect = CGRectMake(0, (scrrenHeight - height) / 2, screenWidth, height);
        [self.previewLayer setFrame:frameRect];
        self.expandOrientation = ISPreviewExpandOrientationPortait;
    }
    else
    {
        CGFloat width = self.imageFrameHeight * scrrenHeight / self.imageFrameWidth;
        CGRect frameRect = CGRectMake((screenWidth - width) / 2, 0, width, scrrenHeight);
        [self.previewLayer setFrame:frameRect];
        self.expandOrientation = ISPreviewExpandOrientationLandscape;
    }
    [rootLayer addSublayer:self.previewLayer];
    [session startRunning];
    
}

- (void)setupBorderView
{
    CGRect borderRect = CGRectMake((self.previewLayer.frame.size.width-PREVIEW_BORDER_WIDTH)/2, (self.previewLayer.frame.size.height-PREVIEW_BORDER_HEIGHT)/2, PREVIEW_BORDER_WIDTH, PREVIEW_BORDER_HEIGHT);
    
    UIView *borderView = [[UIView alloc] init];
    borderView.frame = borderRect;
    borderView.layer.borderColor = [UIColor redColor].CGColor;
    borderView.layer.borderWidth = 1.0f;
    borderView.layer.masksToBounds = YES;
    self.borderView = borderView;
    [self.previewLayer addSublayer:borderView.layer];
}


- (void)setupOtherThings
{
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(didTap:)];
    [self.view addGestureRecognizer:tapGesture];
}

- (void) didTap:(UITapGestureRecognizer *) tapGesture
{
    if (tapGesture.state == UIGestureRecognizerStateBegan)
    {
        [self focus];
    }
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
    
    CFRetain(sampleBuffer);
    dispatch_async(self.videoDataOutputQueue, ^
                   {
                       CGFloat scale = self.imageFrameWidth / self.previewLayer.frame.size.height;
                       CGRect rect = CGRectMake(self.borderView.frame.origin.y * scale, self.borderView.frame.origin.x * scale, self.borderView.frame.size.height * scale, self.borderView.frame.size.width * scale);
                       
                       @autoreleasepool {
                           [[ISVehicleLicensePreviewController sharedISOpenSDKController] detectCardWithOutputSampleBuffer:sampleBuffer cardRect:rect detectCardFinishHandler:^(int result, NSArray *borderPointsArray) {
                               if (result > 0)
                               {
                                   dispatch_async(dispatch_get_main_queue(), ^
                                                  {
                                                      [self.cardBorderLayer removeFromSuperlayer];
                                                      self.cardBorderLayer = nil;
                                                      self.borderView.layer.borderWidth = 10.0f;
                                                      [self drawBorderRectWithBorderPoints:borderPointsArray onCardBorderLayer:YES];
                                                  });
                               }
                               else
                               {
                                   dispatch_async(dispatch_get_main_queue(), ^
                                                  {
                                                      self.borderView.layer.borderWidth = 1.0f;
                                                      [self.cardBorderLayer removeFromSuperlayer];
                                                      self.cardBorderLayer = nil;
                                                  });
                                   
                               }
                               
                           } recognizeCardFinishHandler:^(NSDictionary *cardInfo) {
                               NSLog(@"%lu", (unsigned long)cardInfo.count);
                               [self.captureSession stopRunning];
                               dispatch_async(dispatch_get_main_queue(), ^{
                                   ISVehicleCardResultViewController *vc = [[ISVehicleCardResultViewController alloc] initWithDictionary:cardInfo];
                                   UIImage *originImage = [cardInfo objectForKey:kOpenSDKCardResultTypeOriginImage];
                                   UIImage *croppedImage = [cardInfo objectForKey:kOpenSDKCardResultTypeImage];
                                   NSLog(@"Result origin image %@, cropped image %@", originImage, croppedImage);
                                   [self presentViewController:vc animated:YES completion:^{
                                       
                                   }];
//                                   [self.navigationController pushViewController:vc animated:YES];
                               });
                               
                           }];
                       }
                       CFRelease(sampleBuffer);
                   });
    
}

- (void)drawBorderRectWithBorderPoints:(NSArray *)borderPoints onCardBorderLayer:(BOOL)flag
{
    if ([borderPoints count] >= 4)
    {
        CGPoint point1 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:0] CGPointValue]];
        CGPoint point2 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:1] CGPointValue]];
        CGPoint point3 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:2] CGPointValue]];
        CGPoint point4 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:3] CGPointValue]];
        
        CAShapeLayer *line = [CAShapeLayer layer];
        UIBezierPath *linePath = [UIBezierPath bezierPath];
        [linePath moveToPoint:point1];
        [linePath addLineToPoint:point2];
        [linePath addLineToPoint:point3];
        [linePath addLineToPoint:point4];
        [linePath addLineToPoint:point1];
        line.path = linePath.CGPath;
        line.fillColor = nil;
        line.opacity = 1.0;
        line.strokeColor = [UIColor blueColor].CGColor;
        if (flag)
        {
            self.cardBorderLayer = line;
        }
        else
        {
            self.cardNumberLayer = line;
        }
        [self.previewLayer addSublayer:line];
    }
}

- (CGPoint)convertBorderPointsInImageToPreviewLayer:(CGPoint )borderPoint
{
    //video orientation is landscape
    CGFloat y = borderPoint.x * self.previewLayer.bounds.size.height / self.imageFrameWidth;
    CGFloat x = self.previewLayer.bounds.size.width - borderPoint.y * self.previewLayer.bounds.size.width / self.imageFrameHeight;
    return CGPointMake(x, y);
}

@end
